package	jp.co.sds_corp.volcano;

import java.awt.Image;
import java.io.File;
import java.util.List;

import javax.media.ConfigureCompleteEvent;
import javax.media.ControllerEvent;
import javax.media.ControllerListener;
import javax.media.DataSink;
import javax.media.EndOfMediaEvent;
import javax.media.Format;
import javax.media.Manager;
import javax.media.MediaLocator;
import javax.media.PrefetchCompleteEvent;
import javax.media.Processor;
import javax.media.RealizeCompleteEvent;
import javax.media.ResourceUnavailableEvent;
import javax.media.control.TrackControl;
import javax.media.datasink.DataSinkErrorEvent;
import javax.media.datasink.DataSinkEvent;
import javax.media.datasink.DataSinkListener;
import javax.media.datasink.EndOfStreamEvent;
import javax.media.protocol.ContentDescriptor;
import javax.media.protocol.DataSource;
import javax.media.protocol.FileTypeDescriptor;

public class JpegImagesToMovie implements ControllerListener, DataSinkListener
{
	String	contentType = FileTypeDescriptor.QUICKTIME;
//	String	contentType = FileTypeDescriptor.MSVIDEO;
//	String	contentType = FileTypeDescriptor.MPEG;

	public void setContentType( String var ) {
		this.contentType = var;
	}

	public void makeMovie( int width, int height, int frameRate,
								List<Image> inFiles, MediaLocator outML )
		throws Exception
	{
		ImageDataSource ids = new ImageDataSource(width, height, frameRate, inFiles);

		Processor p;

		p = Manager.createProcessor(ids);
		p.addControllerListener(this);

		// Put the Processor into configured state so we can set
		// some processing options on the processor.
		p.configure();
		if ( !waitForState( p, Processor.Configured )) {
			throw( new Exception( "Failed to configure the processor." ));
		}

		// Set the output content descriptor to QuickTime. 
		ContentDescriptor desc = new ContentDescriptor( contentType );
		p.setContentDescriptor( desc );
System.err.println( "output=" + outML );
System.err.println( "ContentType=" + contentType );

		// Query for the processor for supported formats.
		// Then set it on the processor.
		TrackControl tcs[] = p.getTrackControls();
		Format f[] = tcs[0].getSupportedFormats();
		if (f == null || f.length <= 0) {
			throw( new Exception( "The mux does not support the input format: "
					+ tcs[0].getFormat()));
		}

		tcs[0].setFormat(f[0]);
System.err.println("Setting the track format to: " + f[0]);

		// We are done with programming the processor.  Let's just
		// realize it.
		p.realize();
		if ( !waitForState( p, Processor.Realized )) {
			throw( new Exception( "Failed to realize the processor." ));
		}

		// Now, we'll need to create a DataSink.
		DataSink dsink;
		dsink = createDataSink(p, outML);
		if ( dsink == null ) {
			throw( new Exception( "Failed to create a DataSink"
							+ " for the given output MediaLocator: " + outML ));
		}

		dsink.addDataSinkListener(this);
		fileDone = false;

System.err.println("start processing...");

		p.start();
		dsink.start();

		// Wait for EndOfStream event.
		waitForFileDone();

		// Cleanup.
		dsink.close();
		p.removeControllerListener(this);
	}


	/**
	 * Create the DataSink.
	 */
	DataSink createDataSink( Processor p, MediaLocator outML )
		throws Exception
	{
		DataSource ds;
		ds = p.getDataOutput();
		if ( ds == null ) {
			throw( new Exception( "processor does not have an output DataSource" ));
		}

		DataSink dsink;
		dsink = Manager.createDataSink(ds, outML);
		dsink.open();

		return dsink;
	}


	Object waitSync = new Object();
	boolean stateTransitionOK = true;

	/**
	 * Block until the processor has transitioned to the given state.
	 * Return false if the transition failed.
	 */
	boolean waitForState(Processor p, int state) {
		synchronized (waitSync) {
			try {
			while (p.getState() < state && stateTransitionOK)
				waitSync.wait();
			} catch (Exception e) {
			}
		}
		return stateTransitionOK;
	}


	/**
	 * Controller Listener.
	 */
	public void controllerUpdate(ControllerEvent evt) {

		if (evt instanceof ConfigureCompleteEvent ||
			evt instanceof RealizeCompleteEvent ||
			evt instanceof PrefetchCompleteEvent) {
			synchronized (waitSync) {
				stateTransitionOK = true;
				waitSync.notifyAll();
			}
		} else if (evt instanceof ResourceUnavailableEvent) {
			synchronized (waitSync) {
				stateTransitionOK = false;
				waitSync.notifyAll();
			}
		} else if (evt instanceof EndOfMediaEvent) {
			evt.getSourceController().stop();
			evt.getSourceController().close();
		}
	}


	Object waitFileSync = new Object();
	boolean fileDone = false;
	boolean fileSuccess = true;

	/**
	 * Block until file writing is done. 
	 */
	boolean waitForFileDone() {
		synchronized (waitFileSync) {
			try {
				while (!fileDone) {
					waitFileSync.wait();
				}
			} catch (Exception e) {}
		}
		return fileSuccess;
	}


	/**
	 * Event handler for the file writer.
	 */
	public void dataSinkUpdate(DataSinkEvent evt) {

		if (evt instanceof EndOfStreamEvent) {
			synchronized (waitFileSync) {
				fileDone = true;
				waitFileSync.notifyAll();
			}
		} else if (evt instanceof DataSinkErrorEvent) {
			synchronized (waitFileSync) {
				fileDone = true;
				fileSuccess = false;
				waitFileSync.notifyAll();
			}
		}
	}

	/**
	 * Create a media locator from the given string.
	 */
	public static MediaLocator createMediaLocator(String url) {
		MediaLocator ml;

		if (url.indexOf(":") > 0 && (ml = new MediaLocator(url)) != null) {
			return ml;
		}

		if (url.startsWith(File.separator)) {
			if ((ml = new MediaLocator("file:" + url)) != null) {
				return ml;
			}
		} else {
			String file = "file:" + System.getProperty("user.dir") + File.separator + url;
			if ((ml = new MediaLocator(file)) != null) {
				return ml;
			}
		}

		return null;
	}

}
